XL320_hole_d = 4;
XL320_hole_space = 18;
XL320_axis_d = 3;
XL320_aligned_hole_offsets = [ -6, 0, 6 ];
XL320_hole_x_offset = 22.5;
XL320_dim = [ 36, 24, 24 ];
XL320_horn_h = 3;
XL320_horn_d = 18;
XL320_axis_offset = 9;

function dist(x) = sqrt(x[0] * x[0] + x[1] * x[1] + x[2] * x[2]);

module tube(d1, d2, t) {
    difference() {
        cylinder(d = d2, h = t);
        cylinder(d = d1, h = t + 1);
    }
}

module XL320_aligned_hole_pos() { translate([ 33 - 9, 0, 0 ]) children(); }

module XL320() color("gray") render() difference() {
    $fn = 32;
    dim = XL320_dim;
    r = 4;
    hole_d = XL320_hole_d;

    translate([ dim[0] / 2 - XL320_axis_offset, 0, 0 ]) hull() {
        cube([ dim[0] - r * 2, dim[1], dim[2] ], center = true);
        cube([ dim[0], dim[1], dim[2] - r * 2 ], center = true);
    }
    rotate([ 90, 0, 0 ]) cylinder(d = XL320_axis_d, h = 100, center = true);

    translate([ XL320_hole_x_offset + 5, 0, 0 ])
        cube([ 10, XL320_hole_space, 100 ], center = true);
    XL320_aligned_hole_pos() {
        for (z = XL320_aligned_hole_offsets)
            translate([ 0, 0, z ]) rotate([ 90, 0, 0 ])
                cylinder(d = hole_d, h = 100, center = true);
    }
}

module XL320_horn_hole_pos() {
    for (a = [0:90:360])
        rotate([ 0, 0, a ]) translate([ 12 / 2, 0, 0 ]) children();
}

module XL320_horn() color("gray") render() {
    $fn = 32;
    translate([ 0, -12, 0 ]) rotate([ 90, 0, 0 ]) difference() {
        cylinder(d = XL320_horn_d, h = XL320_horn_h);

        XL320_horn_hole_pos()
            cylinder(d = XL320_hole_d, h = 100, center = true);

        cylinder(d = XL320_axis_d, h = 100, center = true);
    }
}

module XL320_horn_adapter(h = 2) {
    t = 2;
    screw_d = 3;

    tube(XL320_horn_d - t * 2, XL320_horn_d, h);
    tube(XL320_horn_d - t * 2 - screw_d * 2 - t * 2,
         XL320_horn_d - screw_d * 2 - t * 2, h);

    XL320_horn_hole_pos() {
        cylinder(d = XL320_hole_d, h = t * 2, center = true);
        tube(screw_d, screw_d + t * 2, h);
    }
}
